@@ -551,8 +551,15 @@ class WordToken(models.Model):
             self.setStrFormants()
         return self.StrVowelF2
 
-    def getImmediatePrevSemPred(self):
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getPreviousContext(window=2) if lookupCat(str(x)) in ['n','v','a','r']]
+    def getPrevSemPred(self,style='A',window='A'):
+        if style == 'A' and window == 'A':
+            if self.PrevSemPred is not None:
+                return self.PrevSemPred
+        if window != 'A':
+            prev_context = self.getPreviousContext(window = int(window))
+        else:
+            prev_context = self.getPreviousContext()
+        context = ['#'.join([str(x),lookupCat(str(x))]) for x in prev_context if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
         cat =lookupCat(str(self))
         if cat == 'NA':
             cat = self.Category.Label
@@ -566,89 +573,22 @@ class WordToken(models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        return sp
-
-    def getImmediateFollSemPred(self):
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getFollowingContext(window=2) if lookupCat(str(x)) in ['n','v','a','r']]
-        cat =lookupCat(str(self))
-        if cat == 'NA':
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        return sp
-
-    def getIntermedPrevSemPred(self):
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getPreviousContext(window=5) if lookupCat(str(x)) in ['n','v','a','r']]
-        cat =lookupCat(str(self))
-        if cat == 'NA':
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        return sp
-
-    def getIntermedFollSemPred(self):
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getFollowingContext(window=5) if lookupCat(str(x)) in ['n','v','a','r']]
-        cat =lookupCat(str(self))
-        if cat == 'NA':
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        return sp
-
-    def getPrevSemPred(self):
-        if self.PrevSemPred is not None:
+        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context,style=style)
+        if style == 'A' and window == 'A':
+            self.PrevSemPred = sp
+            self.save()
             return self.PrevSemPred
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getPreviousContext() if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
-        cat =lookupCat(str(self))
-        if cat == 'NA':
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        self.PrevSemPred = sp
-        self.save()
-        return self.PrevSemPred
+        return sp
 
-    def getFollSemPred(self):
-        if self.FollSemPred is not None:
-            return self.FollSemPred
-        context = ['#'.join([str(x),lookupCat(str(x))]) for x in self.getFollowingContext() if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
+    def getFollSemPred(self,style='A',window='A'):
+        if style == 'A' and window == 'A':
+            if self.FollSemPred is not None:
+                return self.FollSemPred
+        if window != 'A':
+            foll_context = self.getFollowingContext(window = int(window))
+        else:
+            foll_context = self.getFollowingContext()
+        context = ['#'.join([str(x),lookupCat(str(x))]) for x in foll_context if lookupCat(str(x)) in ['n','v','a','r'] and str(x) != str(self)]
         cat =lookupCat(str(self))
         if cat == 'NA':
             cat = self.Category.Label
@@ -662,10 +602,12 @@ class WordToken(models.Model):
                 cat = 'r'
             else:
                 cat = 'NA'
-        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context)
-        self.FollSemPred = sp
-        self.save()
-        return self.FollSemPred
+        sp = getSemanticRelatedness('#'.join([self.WordType.Label,cat]),context,style=style)
+        if style == 'A' and window == 'A':
+            self.FollSemPred = sp
+            self.save()
+            return self.FollSemPred
+        return sp
 
     def setStrFormants(self,formants=None):
         if formants is not None:
@@ -775,7 +717,7 @@ class WordToken(models.Model):
             if prev.WordType.isWord():
                 context.append(prev)
             durSum += prev.getDuration()
-            if prev.afterPause():
+            if window == 'auto' and prev.afterPause():
                 break
             prev = prev.getPreviousWord()
         return reversed(context)
@@ -794,7 +736,7 @@ class WordToken(models.Model):
             if foll.WordType.isWord():
                 context.append(foll)
             durSum += foll.getDuration()
-            if foll.beforePause():
+            if window == 'auto' and foll.beforePause():
                 break
             foll = foll.getFollowingWord()
         return context
@@ -1079,18 +1021,16 @@ class WordToken(models.Model):
             if 'PhonoStatsSPhoneProb' not in outline:
                 outline['PhonoStatsSPhoneProb'],outline['PhonoStatsBiPhoneProb'] = getPhonotacticProb(self.WordType.getStrUR(),use_blick=False,no_stress=True)
 
-        if form['prev_sem_pred']:
-            if 'PrevSemPred' not in outline:
-                outline['PrevSemPred'] = self.getPrevSemPred()
-            if 'ImmedPrevSemPred' not in outline:
-                outline['ImmedPrevSemPred'] = self.getImmediatePrevSemPred()
-                outline['IntermedPrevSemPred'] = self.getIntermedPrevSemPred()
-        if form['foll_sem_pred']:
-            if 'FollSemPred' not in outline:
-                outline['FollSemPred'] = self.getFollSemPred()
-            if 'ImmedFollSemPred' not in outline:
-                outline['ImmedFollSemPred'] = self.getImmediateFollSemPred()
-                outline['IntermedFollSemPred'] = self.getIntermedFollSemPred()
+        for w in form['sem_pred_window']:
+            for s in form['sem_pred_style']:
+                if form['prev_sem_pred']:
+                    label = '%sWindowPrev%sSemPred' %(w,s)
+                    if label not in outline:
+                        outline[label] = self.getPrevSemPred(window = w,style=s)
+                if form['foll_sem_pred']:
+                    label = '%sWindowFoll%sSemPred' %(w,s)
+                    if label not in outline:
+                        outline[label] = self.getFollSemPred(window = w,style=s)
         if form['pause_dist']:
             if 'DistFollPause' not in outline:
                 outline['DistFollPause'] = self.getFollDistToPause()
